
/*
Copyright (c) 2010 Donatien Garnier (donatiengar [at] gmail [dot] com)
 
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef ZG2100NETIF_H
#define ZG2100NETIF_H

struct netif;

#include "mbed.h"

#include "if/lwip/LwipNetIf.h"
#include "if/net/net.h"

#include "drv/zg2100/zg_defs.h"

typedef zg_err ZG2100Err;

class ZG2100NetIf : public LwipNetIf
{
public:
  ZG2100NetIf( PinName mosi, PinName miso, PinName sclk, PinName cs, PinName intp, PinName nrst ); //W/ DHCP
  ZG2100NetIf( PinName mosi, PinName miso, PinName sclk, PinName cs, PinName intp, PinName nrst,
               IpAddr ip, IpAddr netmask, IpAddr gateway, IpAddr dns ); //W/o DHCP
  virtual ~ZG2100NetIf();
  
  void init();
  
  void setSsid(const char* ssid);

  void setWepKey(const byte* key, int len);

  void setWpaPass(const char* pass);

  void setPskKey(const byte* key, int len);

  ZG2100Err connect(ZG_BSS_TYPE type, ZG_SECURITY security);

  void disconnect();
  
  ZG2100Err setup(int timeout_ms = 15000);

  virtual void poll();

private:
  void waitReady();

  SPI m_spi;
  DigitalOut m_cs;
  InterruptIn m_intp;
  DigitalOut m_nrst;

  Timer m_ethArpTimer;
  Timer m_dhcpCoarseTimer;
  Timer m_dhcpFineTimer;
    
  bool m_useDhcp;

  netif* m_pNetIf;
  
  IpAddr m_netmask;
  IpAddr m_gateway;
  
  const char* m_hostname;
  
};

#endif

